/***************************************************************************
 *          read_images_nifti.cpp                                          *
 *          read header and data from images                               *
 *          4/9/09                                                         *
 ***************************************************************************/

#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <blitz/array.h>
#include <nifti1.h>
#include <nifti1_io.h>
#include <znzlib.h>

#include <omp.h>

#include "copy_nim.h"

#define MIN_HEADER_SIZE 348
#define NII_HEADER_SIZE 352

using namespace std;
BZ_USING_NAMESPACE(blitz)

int read_image_hdr(ofstream *toFile,char fname[200], nifti_image *nim/*int *x_dim, int *y_dim, int *z_dim, int *zz_dim, float *x_res, float *y_res, float *z_res, float *zz_res,int *data_type, int *bits_pix, float *scale, float *intercept, float *offset,int *byte_swap*/) {
  nifti_image *nim_new;

  nim_new=nifti_image_read(fname,0);
  copy_nim(nim_new,nim);

  if (nim_new!= NULL) {
    *(toFile) << "nifti struct fname " << nim->fname << endl;
    *(toFile) << "nifti struct iname " << nim->iname << endl;

    *(toFile) <<"x dim " << nim-> dim[1] << endl;
    *(toFile) <<"y dim " << nim-> dim[2] << endl;
    *(toFile) << "z dim " << nim-> dim[3] << endl;
    *(toFile) << "zz dim " << nim-> dim[4] << endl;
    *(toFile) << "x resolution " << nim->pixdim[1] << endl;
    *(toFile) << "y resolution " << nim->pixdim[2] << endl;
    *(toFile) << "z resolution " << nim->pixdim[3] << endl;
    *(toFile) << "zz resolution" << nim->pixdim[4] << endl;
    *(toFile) << "data type " << nim->datatype << endl;
    *(toFile) << "bits per pixel " << nim->nbyper << endl;
    *(toFile) << "scale " << nim->scl_slope << endl;
    *(toFile) << "intercept " << nim->scl_inter << endl;
  }
  else { *(toFile) << "Error reading file " << fname[0] << endl;
  }

  return(0);
}


int read_images(ofstream *toFile, char fname[500][200], Array<float,5> images) {
  int a,b,c,d,e;

  nifti_image *nim;

  *(toFile) << "in read_images function" << endl;
#pragma omp parallel for private(a,b,c,d,e,nim)
  for(e=0; e<images.extent(fifthDim); e++) {
    nim=nifti_image_read(fname[e],1);
    if (nim== NULL) {
      *(toFile) << "ERROR: cannot open file " << fname[e] << endl;
      cout << "ERROR: cannot open file " << fname[e] << ", exiting" << endl;
      exit(1);
    }
    cout << "reading " << fname[e] << endl;
    switch(nim->datatype) {
    case 2:
      unsigned char *cptr;
      cptr=(unsigned char *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*cptr);
	      cptr++;
	    }
	  }
	}
      }
      break;
    case 4:
      short *sptr;
      sptr=(short *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*sptr);
	      sptr++;
	    }
	  }
	}
      }
      break;
    case 8:
      int *iptr;
      iptr=(int *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*iptr);
	      iptr++;
	    }
	  }
	}
      }
      break;
    case 16:
      float *fptr;
      fptr=(float *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*fptr);
	      fptr++;
	    }
	  }
	}
      }
      break;
    case 64:
      double *dptr;
      dptr=(double *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*dptr);
	      dptr++;
	    }
	  }
	}
      }
      break;
    default:
      short *defptr;
      defptr=(short *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*defptr);
	      defptr++;
	    }
	  }
	}
      }
      break;
    }
    free(nim->data);
  }
  return(0);
}

int read_images_multi(ofstream *toFile, char fname[500][10][200], Array<float,5> images,int sep_files,int num_dep_var) {
  int a,b,c,d,e;

  nifti_image *nim;

  *(toFile) << "in read_images multi function" << endl;
  // assume that if *not* using separate files, zz dimension has the dependent variables
  if(sep_files==0) {
#pragma omp parallel for private(a,b,c,d,nim)
  for(e=0; e<images.extent(fifthDim); e++) {
    nim=nifti_image_read(fname[e][0],1);
    if (nim== NULL) {
      *(toFile) << "ERROR: cannot open file " << fname[e][0] << endl;
      cout << "ERROR: cannot open file " << fname[e][0] << ", exiting" << endl;
      exit(1);
    }
    if(nim->dim[4]!=images.extent(fourthDim)) {
	 *(toFile) << "ERROR: number of dependent variables does not equal zz dim in image file" << endl;
         cout << "ERROR: number of dependent variables does not equal zz dim in image file, exiting" << endl;
	 exit(1);
    }
    switch(nim->datatype) {
    case 2:
      unsigned char *cptr;
      cptr=(unsigned char *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*cptr);
	      cptr++;
	    }
	  }
	}
      }
      break;
    case 4:
      short *sptr;
      sptr=(short *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*sptr);
	      sptr++;
	    }
	  }
	}
      }
      break;
    case 8:
      int *iptr;
      iptr=(int *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*iptr);
	      iptr++;
	    }
	  }
	}
      }
      break;
    case 16:
      float *fptr;
      fptr=(float *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*fptr);
	      fptr++;
	    }
	  }
	}
      }
      break;
    case 64:
      double *dptr;
      dptr=(double *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*dptr);
	      dptr++;
	    }
	  }
	}
      }
      break;
    default:
      short *defptr;
      defptr=(short *)(nim->data);
      for(d=0; d<images.extent(fourthDim); d++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*defptr);
	      defptr++;
	    }
	  }
	}
      }
      break;
    }
    free(nim->data);
  }

    //    *(toFile) << "image " << e << " value 10 12 15 " << images(10,12,15,1,e) << endl;
  }
  else {
    // assume that data of interest is first in file (if zdim != 1)
#pragma omp parallel for private(a,b,c,d,nim)
  for(e=0; e<images.extent(fifthDim); e++) {
    for(d=0; d<images.extent(fourthDim); d++) {
      nim=nifti_image_read(fname[e][d],1);
      if (nim== NULL) {
	*(toFile) << "ERROR: cannot open file " << fname[e][d] << endl;
        cout << "ERROR: cannot open file " << fname[e][d] << ", exiting" << endl;
	exit(1);
      }
      if(nim->dim[4]!=1) {
	*(toFile) << "ERROR: zdim in image file is not 1; do not know which image to use for dependent variable, quitting" << endl;
        cout << "ERROR: zdim in image file is not 1; do not know which image to use for dependent variable, quitting" << endl;
	exit(1);
      }
      switch(nim->datatype) {
      case 2:
	unsigned char *cptr;
	cptr=(unsigned char *)(nim->data);
	  for(c=0; c<images.extent(thirdDim); c++) {
	    for(b=0; b<images.extent(secondDim); b++) {
	      for(a=0; a<images.extent(firstDim); a++) {
		images(a,b,c,d,e)=(float)(*cptr);
		cptr++;
	      }
	    }
	  }
	break;
    case 4:
      short *sptr;
      sptr=(short *)(nim->data);
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*sptr);
	      sptr++;
	    }
	  }
	}
      break;
    case 8:
      int *iptr;
      iptr=(int *)(nim->data);
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*iptr);
	      iptr++;
	    }
	  }
	}
      break;
    case 16:
      float *fptr;
      fptr=(float *)(nim->data);
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*fptr);
	      fptr++;
	    }
	  }
	}
      break;
    case 64:
      double *dptr;
      dptr=(double *)(nim->data);
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*dptr);
	      dptr++;
	    }
	  }
	}
      break;
    default:
      short *defptr;
      defptr=(short *)(nim->data);
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,d,e)=(float)(*defptr);
	      defptr++;
	    }
	  }
	}
      break;
      }
    }
    free(nim->data);
  }
  }
  return(0);
}


int read_image(ofstream *toFile, char fname[200], Array<float,4> image) {
  int a,b,c,d;
  nifti_image *nim;

  *(toFile) << "in read_image function" << endl;

    nim=nifti_image_read(fname,1);
    if (nim== NULL) {
      *(toFile) << "ERROR: cannot open file " << fname << endl;
      cout << "ERROR: cannot open file " << fname << ", exiting" << endl;
      exit(1);
    }
    switch(nim->datatype) {
    case 2:
      unsigned char *cptr;
      cptr=(unsigned char *)(nim->data);
      for(d=0; d<image.extent(fourthDim); d++) {
	for(c=0; c<image.extent(thirdDim); c++) {
	  for(b=0; b<image.extent(secondDim); b++) {
	    for(a=0; a<image.extent(firstDim); a++) {
	      image(a,b,c,d)=(float)(*cptr);
	      cptr++;
	    }
	  }
	}
      }
      break;
    case 4:
      short *sptr;
      sptr=(short *)(nim->data);
      for(d=0; d<image.extent(fourthDim); d++) {
	for(c=0; c<image.extent(thirdDim); c++) {
	  for(b=0; b<image.extent(secondDim); b++) {
	    for(a=0; a<image.extent(firstDim); a++) {
	      image(a,b,c,d)=(float)(*sptr);
	      sptr++;
	    }
	  }
	}
      }
      break;
    case 8:
      int *iptr;
      iptr=(int *)(nim->data);
      for(d=0; d<image.extent(fourthDim); d++) {
	for(c=0; c<image.extent(thirdDim); c++) {
	  for(b=0; b<image.extent(secondDim); b++) {
	    for(a=0; a<image.extent(firstDim); a++) {
	      image(a,b,c,d)=(float)(*iptr);
	      iptr++;
	    }
	  }
	}
      }
      break;
    case 16:
      float *fptr;
      fptr=(float *)(nim->data);
      for(d=0; d<image.extent(fourthDim); d++) {
	for(c=0; c<image.extent(thirdDim); c++) {
	  for(b=0; b<image.extent(secondDim); b++) {
	    for(a=0; a<image.extent(firstDim); a++) {
	      image(a,b,c,d)=(float)(*fptr);
	      fptr++;
	    }
	  }
	}
      }
      break;
    case 64:
      double *dptr;
      dptr=(double *)(nim->data);
      for(d=0; d<image.extent(fourthDim); d++) {
	for(c=0; c<image.extent(thirdDim); c++) {
	  for(b=0; b<image.extent(secondDim); b++) {
	    for(a=0; a<image.extent(firstDim); a++) {
	      image(a,b,c,d)=(float)(*dptr);
	      dptr++;
	    }
	  }
	}
      }
      break;
    default:
      short *defptr;
      defptr=(short *)(nim->data);
      for(d=0; d<image.extent(fourthDim); d++) {
	for(c=0; c<image.extent(thirdDim); c++) {
	  for(b=0; b<image.extent(secondDim); b++) {
	    for(a=0; a<image.extent(firstDim); a++) {
	      image(a,b,c,d)=(float)(*defptr);
	      defptr++;
	    }
	  }
	}
      }
      break;
    }
    free(nim->data);
  return(0);
}

int read_ind_images(ofstream *toFile, char fname[500][10][200], Array<float,6> images) {
  int a,b,c,d,e,f;

  nifti_image *nim;

  *(toFile) << "in read_ind_images function" << endl;
#pragma omp parallel for private(a,b,c,d,f,nim)
  for(e=0; e<images.extent(sixthDim); e++) {
    for(d=0; d<images.extent(fifthDim); d++) {
      nim=nifti_image_read(fname[e][d],1);
      if (nim== NULL) {
	*(toFile) << "ERROR: cannot open file " << fname[e][d] << endl;
        cout << "ERROR: cannot open file " << fname[e][d] << ", exiting" << endl;
	exit(1);
      }
      switch(nim->datatype) {
      case 2:
	unsigned char *cptr;
	cptr=(unsigned char *)(nim->data);
	for(f=0; f<images.extent(fourthDim); f++) {
	for(c=0; c<images.extent(thirdDim); c++) {
	  for(b=0; b<images.extent(secondDim); b++) {
	    for(a=0; a<images.extent(firstDim); a++) {
	      images(a,b,c,f,d,e)=(float)(*cptr);
	      cptr++;
	    }
	  }
	}
	}
	break;
    case 4:
      short *sptr;
      sptr=(short *)(nim->data);
      for(f=0; f<images.extent(fourthDim); f++) {
      for(c=0; c<images.extent(thirdDim); c++) {
	for(b=0; b<images.extent(secondDim); b++) {
	  for(a=0; a<images.extent(firstDim); a++) {
	    images(a,b,c,f,d,e)=(float)(*sptr);
	    sptr++;
	  }
	}
      }
      }
      break;
    case 8:
      int *iptr;
      iptr=(int *)(nim->data);
      for(f=0; f<images.extent(fourthDim); f++) {
      for(c=0; c<images.extent(thirdDim); c++) {
	for(b=0; b<images.extent(secondDim); b++) {
	  for(a=0; a<images.extent(firstDim); a++) {
	    images(a,b,c,f,d,e)=(float)(*iptr);
	    iptr++;
	  }
	}
      }
      }
      break;
    case 16:
      float *fptr;
      fptr=(float *)(nim->data);
      for(f=0; f<images.extent(fourthDim); f++) {
      for(c=0; c<images.extent(thirdDim); c++) {
	for(b=0; b<images.extent(secondDim); b++) {
	  for(a=0; a<images.extent(firstDim); a++) {
	    images(a,b,c,f,d,e)=(float)(*fptr);
	    fptr++;
	  }
	}
      }
      }
      break;
    case 64:
      double *dptr;
      dptr=(double *)(nim->data);
      for(f=0; f<images.extent(fourthDim); f++) {
      for(c=0; c<images.extent(thirdDim); c++) {
	for(b=0; b<images.extent(secondDim); b++) {
	  for(a=0; a<images.extent(firstDim); a++) {
	    images(a,b,c,f,d,e)=(float)(*dptr);
	    dptr++;
	  }
	}
      }
      }
      break;
    default:
      short *defptr;
      defptr=(short *)(nim->data);
      for(f=0; f<images.extent(fourthDim); f++) {
      for(c=0; c<images.extent(thirdDim); c++) {
	for(b=0; b<images.extent(secondDim); b++) {
	  for(a=0; a<images.extent(firstDim); a++) {
	    images(a,b,c,f,d,e)=(float)(*defptr);
	    defptr++;
	  }
	}
      }
      }
      break;
      }
      free(nim->data);
    
    }
  }
  return(0);
  }
