/*************************************************************************
 *  read_model_matrices.cpp                                                  *
 *  read in design matrices from input file                              
 *************************************************************************/

#include <blitz/array.h>
#include <fstream>


using namespace std;

BZ_USING_NAMESPACE(blitz)

  int read_design_mat(ofstream *toFile,ifstream *fromFile,Array<float,2> X1,Array<float,2> X2,Array<float,2> X,int norm, Array<float,1> normVal, char fname[500][200]) {

 int a, b;
  *(toFile) << "printing out design matrices" << endl;
  for(a=0; a<X1.extent(firstDim); a++) {
    *(fromFile) >> fname[a];
    *(toFile) << fname[a] << "\t";
    for(b=0; b<X2.extent(secondDim); b++) {
      *(fromFile) >> X2(a,b);
      X(a,b)=X2(a,b);
      *(toFile) << X2(a,b) << "\t";
    }
    for(b=0; b<X1.extent(secondDim); b++) {
      *(fromFile) >> X1(a,b);
      X(a,b+X2.extent(secondDim))=X1(a,b);
      *(toFile) << X1(a,b) << "\t";
    }
    *(toFile) << "\t\t\t";
    // if normalizing by some value, included in input file
    if(norm==1) {
       *(fromFile) >> normVal(a);
       *(toFile) << normVal(a) << "\t";
    }
    *(toFile) << endl;

  }
  return(0);
}

int read_design_mat_sp(ofstream *toFile,ifstream *fromFile,Array<float,2> X1,Array<float,2> X2,Array<float,2> X,int norm, Array<float,1> normVal, char fname[500][200],char fnamesp[500][10][200],int num_spvar_indep_var) {

 int a, b;
 // cout << "num spatailly varying variables " << num_spvar_indep_var << endl;
  *(toFile) << "printing out design matrices" << endl;
  for(a=0; a<X1.extent(firstDim); a++) {
    *(fromFile) >> fname[a];
    *(toFile) << fname[a] << "\t";
    for(b=0; b<X2.extent(secondDim); b++) {
      *(fromFile) >> X2(a,b);
      X(a,b)=X2(a,b);
      *(toFile) << X2(a,b) << "\t";
    }
    for(b=0; b<X1.extent(secondDim)-num_spvar_indep_var; b++) {
      *(fromFile) >> X1(a,b);
      X(a,b+X2.extent(secondDim))=X1(a,b);
      *(toFile) << X1(a,b) << "\t";
    }
    for(b=0; b<num_spvar_indep_var; b++) {
      *(fromFile) >> fnamesp[a][b];
      // fill spatially varying x with a dummy_var to start
      X(a,b+X2.extent(secondDim)+X1.extent(secondDim)-num_spvar_indep_var)=0.;
      *(toFile) << fnamesp[a][b] << endl;
    }
    // if normalizing by some value, included in input file
    if(norm==1) {
       *(fromFile) >> normVal(a);
       *(toFile) << normVal(a) << "\t";
    }
    *(toFile) << endl;
  }
  return(0);
}

int read_design_mat_multi(ofstream *toFile,ifstream *fromFile,Array<float,2> X1,Array<float,2> X2,Array<float,2> X,int norm, Array<float,1> normVal, char fname[500][10][200],int sep_files,int num_dep_var) {

 int a, b;
  *(toFile) << "printing out design matrices" << endl;
  //  cout << "sep_files " << sep_files << endl;
  //  cout << "num_dep_var " << num_dep_var << endl;
  for(a=0; a<X1.extent(firstDim); a++) {
    if(sep_files==1) {
      for(b=0; b<num_dep_var; b++) {
	*(fromFile) >> fname[a][b];
	*(toFile) << fname[a][b] << "\t";
      }
    }
    else {
      *(fromFile) >> fname[a][0];
      *(toFile) << fname[a][0] << "\t";
    }
    for(b=0; b<X2.extent(secondDim); b++) {
      *(fromFile) >> X2(a,b);
      X(a,b)=X2(a,b);
      *(toFile) << X2(a,b) << "\t";
    }
    for(b=0; b<X1.extent(secondDim); b++) {
      *(fromFile) >> X1(a,b);
      X(a,b+X2.extent(secondDim))=X1(a,b);
      *(toFile) << X1(a,b) << "\t";
    }
    *(toFile) << "\t\t\t";
    // if normalizing by some value, included in input file
    if(norm==1) {
       *(fromFile) >> normVal(a);
       *(toFile) << normVal(a) << "\t";
    }
    *(toFile) << endl;

  }
  return(0);
}

int read_contrast_mat(ofstream *toFile,ifstream *fromFile,Array<float,2> C1) {
  int a, b;
  for(a=0; a<C1.extent(firstDim); a++) {
    for(b=0; b<C1.extent(secondDim); b++) {
      *(fromFile) >> C1(a,b);
      *(toFile) << C1(a,b) << "\t";
    }
    *(toFile) << endl;
  }
  return(0);
}

int read_design_mat_multi_static_sp(ofstream *toFile,ifstream *fromFile,Array<float,2> X1,Array<float,2> X2,Array<float,2> X,int norm, Array<float,1> normVal, Array<float,2> Y,int num_dep_var,char fnamesp[500][10][200],int num_spvar_indep_var) {

 int a, b;
  *(toFile) << "printing out design matrices" << endl;
  for(a=0; a<X1.extent(firstDim); a++) {
    for(b=0; b<num_dep_var; b++) {
      *(fromFile) >> Y(b,a);
      *(toFile) << Y(b,a) << "\t";
    }
    for(b=0; b<X2.extent(secondDim); b++) {
      *(fromFile) >> X2(a,b);
      X(a,b)=X2(a,b);
      *(toFile) << X2(a,b) << "\t";
    }
    for(b=0; b<X1.extent(secondDim)-num_spvar_indep_var; b++) {
      *(fromFile) >> X1(a,b);
      X(a,b+X2.extent(secondDim))=X1(a,b);
      *(toFile) << X1(a,b) << "\t";
    }
    for(b=0; b<num_spvar_indep_var; b++) {
      *(fromFile) >> fnamesp[a][b];
      // fill spatially varying x with a dummy_var to start
      X(a,b+X2.extent(secondDim)+X1.extent(secondDim)-num_spvar_indep_var)=0.;
      *(toFile) << fnamesp[a][b] << endl;
    }
    if(norm==1) {
       *(fromFile) >> normVal(a);
       *(toFile) << normVal(a) << "\t";
    }
    *(toFile) << endl;
  }
  return(0);
}

int read_design_mat_static_sp(ofstream *toFile,ifstream *fromFile,Array<float,2> X1,Array<float,2> X2,Array<float,2> X,int norm, Array<float,1> normVal, Array<float,2> Y,char fnamesp[500][10][200],int num_spvar_indep_var) {

  int a, b;
  *(toFile) << "printing out design matrices" << endl;
  for(a=0; a<X1.extent(firstDim); a++) {
      *(fromFile) >> Y(0,a);
      *(toFile) << Y(0,a) << "\t";
    for(b=0; b<X2.extent(secondDim); b++) {
      *(fromFile) >> X2(a,b);
      X(a,b)=X2(a,b);
      *(toFile) << X2(a,b) << "\t";
    }
    for(b=0; b<X1.extent(secondDim)-num_spvar_indep_var; b++) {
      *(fromFile) >> X1(a,b);
      X(a,b+X2.extent(secondDim))=X1(a,b);
      *(toFile) << X1(a,b) << "\t";
    } 
    for(b=0; b<num_spvar_indep_var; b++) {
      *(fromFile) >> fnamesp[a][b];
      // fill spatially varying x with a dummy_var to start
      X(a,b+X2.extent(secondDim)+X1.extent(secondDim)-num_spvar_indep_var)=0.;
      *(toFile) << fnamesp[a][b] << endl;
      } 
    if(norm==1) {
       *(fromFile) >> normVal(a);
       *(toFile) << normVal(a) << "\t";
    }
    *(toFile) << endl;
 } 
 return(0);
}
