/**********************************************************************
 * statmap2.cpp             Rewrite of statmap
 **********************************************************************/

#include <blitz/array.h>
#include <fstream>

#include "uni_analysis.h"
#include "multi_analysis.h"
using namespace std;

BZ_USING_NAMESPACE(blitz)

  int main(int argc, char* argv[]) {

  int num_dep_var, num_spvar_indep_var, fin_OK;
  ofstream toFile(argv[2], ios::app);
  if(!toFile){ 
    cout << "unable to open output file, " << argv[2] << ", exiting" << endl;
    exit(1);
  }
  toFile << "now opening " << argv[2] << endl;
  toFile << "now opening " << argv[1] << endl;
  ifstream fromFile(argv[1]);


  // read design parameters from file
  if (!fromFile) {
    toFile << "unable to open input file, " << argv[1] << endl;
    cout << "unable to open input file, " << argv[1] << ", exiting" << endl;
    exit(1);
  }
  else {
    fromFile >> num_dep_var;
    fromFile >> num_spvar_indep_var;
  }

  if(num_dep_var==1) {
    if(num_spvar_indep_var>0) {
      fin_OK=uni_analysis_spvar(&toFile, &fromFile,num_spvar_indep_var);
    }
    else {
      fin_OK=uni_analysis(&toFile, &fromFile);
    }
    if(fin_OK==0) {
        toFile << "univariate analysis finished " <<endl;
        cout << "univariate analysis finished" << endl;
    }
    else {
        toFile << "returned error in univariate analysis " << fin_OK << endl;
        cout << "returned error in univariate analysis, exiting" << endl;
        exit(1);
    }
  }
  else if(num_dep_var>1) {
    if(num_spvar_indep_var>0) {
      fin_OK=multi_analysis_spvar(&toFile, &fromFile,num_spvar_indep_var);
    }
    else {
      fin_OK=multi_analysis(&toFile, &fromFile,num_dep_var);
    }
    if(fin_OK==0) {
        toFile << "multivariate analysis finished " <<endl;
        cout << "multivariate analysis finished " << endl;
    }
    else {
        toFile << "returned error in multivariate analysis " << fin_OK << endl;
        cout << "returned error in multivariate analysis, exiting" << endl;
        exit(1);
    }
  }
  else {
    num_dep_var=-1*num_dep_var;
    if(num_dep_var==1) {
      if(num_spvar_indep_var>0) {
	fin_OK=uni_analysis_staticdv_spvar(&toFile, &fromFile,num_spvar_indep_var);
	if(fin_OK!=0) {
            toFile << "returned error in analysis of spatially invariant dependent variable " << endl;
            cout << "returned error in analysis of spatially invariant dependent variable, exiting" << endl;
            exit(1);
        }
      }
      else {
          cout << "no spatially varying variables, use conventional statistical analysis program " << endl;
          cout << "exiting" << endl;
          exit(1);
      }
    }
    else {
      if(num_spvar_indep_var>0) {
	fin_OK=multi_analysis_staticdv_spvar(&toFile,&fromFile,num_dep_var,num_spvar_indep_var);
	if(fin_OK==0) {
            toFile << "analysis on multiple spatially invariant dependent variables finished " << endl;
            cout << "analysis on multiple spatially invariant dependent variables finished " << endl;
        }
	else {
            toFile << "returned error in analysis of multiple spatially invariant dependent variables " << endl;
            cout << "returned error in analysis of multiple spatially invariant dependent variables, exiting" << endl;
            exit(1);
        }
      }
      else {
          cout << "no spatially varying variables, use conventional statistical analsysis program " << endl;
          cout << "exiting" << endl;
          exit(1);
      }
    }
  }

  toFile.close();
  fromFile.close();
  return(1);
}  
