/**********************************************************************************************************
 *             write_images.cpp                                                                           *
 *             5/19/2008                                                                                  *
 **********************************************************************************************************/

#include <fstream>
#include <blitz/array.h>
#include <string.h>
#include <nifti1.h>
#include <nifti1_io.h>
#include <znzlib.h>

#include "copy_nim.h"

using namespace std;
BZ_USING_NAMESPACE(blitz)

int writeBetaImage(ofstream *toFile, Array<float, 5> Bimage, nifti_image *nim_orig /*float x_res, float y_res, float z_res, float zz_res*/, char templateName[200],char basename[200]) {

  char newname[200], tmpstr[10],tag[2];
  int a,b,c,d,e;
  float *ptr;
  strcpy(tmpstr,".nii");

  int newdim[8];
  nifti_image *nim;
  newdim[0]=4;
  newdim[1]=Bimage.extent(firstDim);
  newdim[2]=Bimage.extent(secondDim);
  newdim[3]=Bimage.extent(thirdDim);
  newdim[4]=Bimage.extent(fourthDim);
  nim=nifti_make_new_nim(newdim,16,1);
  copy_nim(nim_orig, nim);
  nim->xyz_units=NIFTI_UNITS_MM;
  nim->intent_code=NIFTI_INTENT_ESTIMATE;
  strcpy(nim->intent_name,"beta estimates");
  if(nim->data == NULL) {
    cout << "ERROR: could not allocate space for data in nifti_image structure" << endl;
    exit(1);
  }
  for(e=0; e<Bimage.extent(fifthDim); e++) {
    strcpy(newname,basename);
    sprintf(tag,"%d",e);
    strcat(newname,tag);
    //    strcat(newname,tmpstr);
    strcat(newname,tmpstr);
    nim->fname=newname;
    nim->iname=newname;
    ptr=(float *)nim->data;
    for(d=0; d<Bimage.extent(fourthDim); d++) {
      for(c=0; c<Bimage.extent(thirdDim); c++) {
	for(b=0; b<Bimage.extent(secondDim); b++) {
	  for (a=0; a<Bimage.extent(firstDim); a++) {
	    *ptr=Bimage(a,b,c,d,e);
	    ptr++;
	  }
	}
      }
    }

    // write nifti file
    nifti_image_write(nim);
    cout << "saved file " << newname << endl;
    *(toFile) << "saved file " << newname << endl;
  }
  free(nim->data);
  return(0);
}

int writeStatImage(ofstream *toFile, Array<float, 3> Stat_image, nifti_image *nim_orig /*float x_res, float y_res, float z_res, float zz_res*/,char templateName[200],char basename[200],float *df1,float *df2) {

  char newname[200], *pch, tmpstr[10];
  int a,b,c;

  pch=strrchr(templateName,'.');
  strcpy(tmpstr,".nii");

  int newdim[8];
  nifti_image *nim;
  newdim[0]=4;
  newdim[1]=Stat_image.extent(firstDim);
  newdim[2]=Stat_image.extent(secondDim);
  newdim[3]=Stat_image.extent(thirdDim);
  newdim[4]=1;
  nim=nifti_make_new_nim(newdim,16,1);
  copy_nim(nim_orig,nim);
  nim->xyz_units=NIFTI_UNITS_MM;
  nim->intent_code=NIFTI_INTENT_FTEST;
  strcpy(nim->intent_name,"Model F-test");
  nim->intent_p1=*df1;
  nim->intent_p2=*df2;
  if(nim->data == NULL) {
    cout << "ERROR: could not allocate space for data in nifti_image structure" << endl;
    exit(1);
  }
  float *ptr=(float *)nim->data;
  strcpy(newname,basename);
  strcat(newname,tmpstr);
  nim->fname=newname;
  nim->iname=newname;
  for(c=0; c<Stat_image.extent(thirdDim); c++) {
    for(b=0; b<Stat_image.extent(secondDim); b++) {
      for (a=0; a<Stat_image.extent(firstDim); a++) {
	*ptr=Stat_image(a,b,c);
	ptr++;
      }
    }
  }
  nifti_image_write(nim);
  cout << "saved file " << newname << endl;
  *(toFile) << "saved file " << newname << endl;
  free(nim->data);
  return(0);
}

int writeStatParamImage(ofstream *toFile,Array<float, 5> Stat_param_image, nifti_image *nim_orig /*float x_res, float y_res, float z_res, float zz_res*/, char templateName[200],char basename[200],float *tdf) {

  char newname[200], tmpstr[10],tag[2];
  int a,b,c,d,e;

  strcpy(tmpstr,".nii");

  int newdim[8];
  nifti_image *nim;
  newdim[0]=4;
  newdim[1]=Stat_param_image.extent(firstDim);
  newdim[2]=Stat_param_image.extent(secondDim);
  newdim[3]=Stat_param_image.extent(thirdDim);
  newdim[4]=Stat_param_image.extent(fourthDim);
  nim=nifti_make_new_nim(newdim,16,1);
  copy_nim(nim_orig,nim);
  nim->xyz_units=NIFTI_UNITS_MM;
  nim->intent_code=NIFTI_INTENT_TTEST;
  nim->intent_p1=*tdf;
  strcpy(nim->intent_name,"t-stat for ind var");
  // allocate memory for data in nifti_image structure
  nim->data=(void *)calloc(1,nim->dim[1]*nim->dim[2]*nim->dim[3]*nim->dim[4]*sizeof(float));
  if(nim->data == NULL) {
    cout << "ERROR: could not allocate space for data in nifti_image structure" << endl;
    exit(1);
  }
  for(e=0; e<Stat_param_image.extent(fifthDim); e++) {
    strcpy(newname,basename);
    sprintf(tag,"%d",e);
    strcat(newname,tag);
    strcat(newname,tmpstr);
    nim->fname=newname;
    nim->iname=newname;
    float *ptr=(float *)nim->data;
    for(d=0; d<Stat_param_image.extent(fourthDim); d++) {
      for(c=0; c<Stat_param_image.extent(thirdDim); c++) {
	for(b=0; b<Stat_param_image.extent(secondDim); b++) {
	  for (a=0; a<Stat_param_image.extent(firstDim); a++) {
	    *ptr=Stat_param_image(a,b,c,d,e);
	    ptr++;
	  }
	}
      }
    }

    // write nifti file
    nifti_image_write(nim);
    cout << "saved file " << newname << endl;
    *(toFile) << "saved file " << newname << endl;
  }
  free(nim->data);
  return(0);
}

int writeR2Image(ofstream *toFile,Array<float, 4> Stat_R2_image, nifti_image *nim_orig /*float x_res, float y_res, float z_res, float zz_res*/,char templateName[200],char basename[200]) {

  char newname[200], tmpstr[10];
  int a,b,c,d;

  strcpy(tmpstr,".nii");

  int newdim[8];
  nifti_image *nim;
  newdim[0]=4;
  newdim[1]=Stat_R2_image.extent(firstDim);
  newdim[2]=Stat_R2_image.extent(secondDim);
  newdim[3]=Stat_R2_image.extent(thirdDim);
  newdim[4]=Stat_R2_image.extent(fourthDim);
  nim=nifti_make_new_nim(newdim,16,1);
  copy_nim(nim_orig,nim);
  nim->xyz_units=NIFTI_UNITS_MM;
  nim->intent_code=NIFTI_INTENT_ESTIMATE;
  strcpy(nim->intent_name,"R2 image");
  if(nim->data == NULL) {
    cout << "ERROR: could not allocate space for data in nifti_image structure" << endl;
    exit(1);
  }
  strcpy(newname,basename);
  strcat(newname,"R2");
  strcat(newname,tmpstr);
  nim->fname=newname;
  nim->iname=newname;
  float *ptr=(float *)nim->data;
  for(d=0; d<Stat_R2_image.extent(fourthDim); d++) {
    for(c=0; c<Stat_R2_image.extent(thirdDim); c++) {
      for(b=0; b<Stat_R2_image.extent(secondDim); b++) {
	for(a=0; a<Stat_R2_image.extent(firstDim); a++) {
	  *ptr=Stat_R2_image(a,b,c,d);
	  ptr++;
	}
      }
    }
  }
  // write nifti file
  nifti_image_write(nim);
  cout << "saved file " << newname << endl;
  *(toFile) << "saved file " << newname << endl;
  free(nim->data);
  return(0);
}

int writeContrastImage(ofstream *toFile,Array<float, 5> Contrast_image, nifti_image *nim_orig /*float x_res, float y_res, float z_res, float zz_res*/,char templateName[200],char basename[200]) {

  char newname[200], tmpstr[10],tag[2];
  int a,b,c,d,e;

  strcpy(tmpstr,".nii");

  int newdim[8];
  nifti_image *nim;
  newdim[0]=4;
  newdim[1]=Contrast_image.extent(firstDim);
  newdim[2]=Contrast_image.extent(secondDim);
  newdim[3]=Contrast_image.extent(thirdDim);
  newdim[4]=Contrast_image.extent(fourthDim);
  nim=nifti_make_new_nim(newdim,16,1);
  copy_nim(nim_orig,nim);
  nim->xyz_units=NIFTI_UNITS_MM;
  nim->intent_code=NIFTI_INTENT_TTEST;
  strcpy(nim->intent_name,"contrast T-stats");
  if(nim->data == NULL) {
    cout << "ERROR: could not allocate space for data in nifti_image structure" << endl;
    exit(1);
  }

  for(e=0; e<Contrast_image.extent(fifthDim); e++) {
    strcpy(newname,basename);
    sprintf(tag,"%d",e);
    strcat(newname,"EstContrast");
    strcat(newname,tag);
    strcat(newname,tmpstr);
    nim->fname=newname;
    nim->iname=newname;
    float *ptr=(float *)nim->data;
    for(d=0; d<Contrast_image.extent(fourthDim); d++) {
      for(c=0; c<Contrast_image.extent(thirdDim); c++) {
	for(b=0; b<Contrast_image.extent(secondDim); b++) {
	  for(a=0; a<Contrast_image.extent(firstDim); a++) {
	    *ptr=Contrast_image(a,b,c,d,e);
	    ptr++;
	  }
	}
      }
    }
    // write nifti file
    nifti_image_write(nim);
    cout << "saved file " << newname << endl;
    *(toFile) << "saved file " << newname << endl;
  }
  free(nim->data);
  return(0);
}

int writeContrastFImage(ofstream *toFile, Array<float, 5> F_contrast_image, nifti_image *nim_orig /*float x_res, float y_res, float z_res, float zz_res*/,char templateName[200],char basename[200],float *cdf1,float *cdf2) {

  char newname[200], tmpstr[10],tag[2];
  int a,b,c,d,e;

  strcpy(tmpstr,".nii");

  int newdim[8];
  nifti_image *nim;
  newdim[0]=4;
  newdim[1]=F_contrast_image.extent(firstDim);
  newdim[2]=F_contrast_image.extent(secondDim);
  newdim[3]=F_contrast_image.extent(thirdDim);
  newdim[4]=F_contrast_image.extent(fourthDim);
  nim=nifti_make_new_nim(newdim,16,1);
  copy_nim(nim_orig,nim);
  nim->xyz_units=NIFTI_UNITS_MM;
  nim->intent_code=NIFTI_INTENT_FTEST;
  strcpy(nim->intent_name,"contrast F-stats");
  nim->intent_p1=*cdf1;
  nim->intent_p2=*cdf2;
  if(nim->data == NULL) {
    cout << "ERROR: could not allocate space for data in nifti_image structure" << endl;
    exit(1);
  }

  for(e=0; e<F_contrast_image.extent(fifthDim); e++) {
    strcpy(newname,basename);
    sprintf(tag,"%d",e);
    strcat(newname,"FContrast");
    strcat(newname,tag);
    strcat(newname,tmpstr);
    nim->fname=newname;
    nim->iname=newname;
    float *ptr=(float *)nim->data;
    for(d=0; d<F_contrast_image.extent(fourthDim); d++) {
      for(c=0; c<F_contrast_image.extent(thirdDim); c++) {
	for(b=0; b<F_contrast_image.extent(secondDim); b++) {
	  for(a=0; a<F_contrast_image.extent(firstDim); a++) {
	    *ptr=F_contrast_image(a,b,c,d,e);
	    ptr++;
	  }
	}
      }
    }
    // write nifti file
    nifti_image_write(nim);
    cout << "saved file " << newname << endl;
    *(toFile) << "save file " << newname << endl;
  }
  free(nim->data);
  return(0);
}

int writeFManova(ofstream *toFile, Array<float, 4> F_manova_image, nifti_image *nim_orig /*float x_res, float y_res, float z_res, float zz_res*/, char templateName[200],char basename[200],const char label[20],float *df1,float *df2) {

  char newname[200], tmpstr[10];
  int a,b,c,d;

  strcpy(tmpstr,".nii");

  int newdim[8];
  nifti_image *nim;
  newdim[0]=4;
  newdim[1]=F_manova_image.extent(firstDim);
  newdim[2]=F_manova_image.extent(secondDim);
  newdim[3]=F_manova_image.extent(thirdDim);
  newdim[4]=F_manova_image.extent(fourthDim);
  nim=nifti_make_new_nim(newdim,16,1);
  copy_nim(nim_orig,nim);
  nim->xyz_units=NIFTI_UNITS_MM;
  nim->intent_code=NIFTI_INTENT_FTEST;
  strcpy(nim->intent_name,"MANOVA F-stats");
  nim->intent_p1=*df1;
  nim->intent_p2=*df2;
  if(nim->data == NULL) {
    cout << "ERROR: could not allocate space for data in nifti_image structure" << endl;
    exit(1);
  }
  strcpy(newname,basename);
  strcat(newname,label);
  strcat(newname,tmpstr);
  nim->fname=newname;
  nim->iname=newname;
  float *ptr=(float *)nim->data;
  for(d=0; d<F_manova_image.extent(fourthDim); d++) {
    for(c=0; c<F_manova_image.extent(thirdDim); c++) {
      for(b=0; b<F_manova_image.extent(secondDim); b++) {
	for(a=0; a<F_manova_image.extent(firstDim); a++) {
	  *ptr=F_manova_image(a,b,c,d);
	  ptr++;
	}
      }
    }
  }
    // write nifti file
  nifti_image_write(nim);
  cout << "saved file " << newname << endl;
  *(toFile) << "saved file " << newname << endl;
  free(nim->data);
  return(0);
}

